# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/featurestore/api/v1/FeatureSetSearch.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1ai/h2o/featurestore/api/v1/FeatureSetSearch.proto\x12\x1a\x61i.h2o.featurestore.api.v1\"?\n\rTextualFilter\x12\r\n\x05\x66ield\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x03(\t\x12\x10\n\x08operator\x18\x03 \x01(\t\"A\n\x0fNumericalFilter\x12\r\n\x05\x66ield\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x10\n\x08operator\x18\x03 \x01(\t\"?\n\rBooleanFilter\x12\r\n\x05\x66ield\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08\x12\x10\n\x08operator\x18\x03 \x01(\t\"\xce\x01\n\x06\x46ilter\x12\x39\n\x04text\x18\x01 \x01(\x0b\x32).ai.h2o.featurestore.api.v1.TextualFilterH\x00\x12>\n\x07numeric\x18\x02 \x01(\x0b\x32+.ai.h2o.featurestore.api.v1.NumericalFilterH\x00\x12<\n\x07\x62oolean\x18\x03 \x01(\x0b\x32).ai.h2o.featurestore.api.v1.BooleanFilterH\x00\x42\x0b\n\tconditionB\x17\x42\x15\x46\x65\x61tureSetSearchProtob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ai.h2o.featurestore.api.v1.FeatureSetSearch_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\025FeatureSetSearchProto'
  _globals['_TEXTUALFILTER']._serialized_start=81
  _globals['_TEXTUALFILTER']._serialized_end=144
  _globals['_NUMERICALFILTER']._serialized_start=146
  _globals['_NUMERICALFILTER']._serialized_end=211
  _globals['_BOOLEANFILTER']._serialized_start=213
  _globals['_BOOLEANFILTER']._serialized_end=276
  _globals['_FILTER']._serialized_start=279
  _globals['_FILTER']._serialized_end=485
# @@protoc_insertion_point(module_scope)
